function [cost,grad] = computeAECostTiedW(theta, visibleSize, hiddenSize, lambda, data)

% visibleSize: the number of input units (probably 64) 
% hiddenSize: the number of hidden units (probably 25) 
% lambda: weight decay parameter
% beta: weight of sparsity penalty term
% data: Our 64x10000 matrix containing the training data.  So, data(:,i) is the i-th training example. 

%% theta2matrix
W1 = reshape(theta(1:hiddenSize*visibleSize), hiddenSize, visibleSize);
b1 = theta(hiddenSize*visibleSize+1:hiddenSize*visibleSize+hiddenSize);
W2 = W1';
b2 = theta(hiddenSize*visibleSize+hiddenSize+1:end);


%% feed forward pass
datasize = size(data);
samples = datasize(2);

% Row-vector to aid in calculation of hidden activations and output values
weightsbuffer = ones(1, samples);

% Calculate activations of hidden and output neurons
hiddeninputs = W1 * data + b1 * weightsbuffer; % hiddensize * numpatches
hiddenvalues = sigmoid( hiddeninputs ); % hiddensize * numpatches

finalinputs = W2 * hiddenvalues + b2 * weightsbuffer; %visiblesize * numpatches
outputs = sigmoid( finalinputs ); %visiblesize * numpatches

errors = outputs - data; %visiblesize * numpatches

cost = sum( sum((errors .* errors)) ./ (2*samples));

%% Back-propagation calculation of gradients
delta3 = errors .* outputs .* (1 - outputs); % Matrix of error terms, visiblesize * numpatches
W2grad = delta3 * transpose(hiddenvalues) / samples; % visiblesize * hiddensize, averaged over all patches
b2grad = delta3 * transpose(weightsbuffer) / samples; % visiblesize * 1, averaged over all patches

delta2 = (transpose(W2) * delta3) .* hiddenvalues .* (1 - hiddenvalues); % hiddensize * numpatches
W1grad = delta2 * transpose(data) / samples; % hiddensize * visiblesize, averaged over all patches
b1grad = delta2 * transpose(weightsbuffer) / samples; % hiddensize * 1, averaged over all patches

% Weight-decay
cost = cost + lambda / 2 * ( sum(sum(W1 .* W1)) +sum( sum(W2 .* W2)) );

W1grad = W1grad + lambda * W1;
W2grad = W2grad + lambda * W2;

W1grad = W1grad + W2grad';

grad = [W1grad(:) ; b1grad(:) ; b2grad(:)];

end

function sigm = sigmoid(x)
  
    sigm = 1 ./ (1 + exp(-x));
end

