function [cost, dW, db] = CalGrad_weighted_gpu(data, label, W, b, AESize, options, wei)
n = length(AESize);
data = gpuArray(data);
label = gpuArray(label);
wei   = gpuArray(wei);

for i=1:n-1
    W{i} = gpuArray(W{i});
    b{i} = gpuArray(b{i});
end
%% feedforward pass

a = cell(1,n);
a{1} = data;
m = size(data , 2);    %sample num

for i=2:n-1
    switch options.activation_function
        case 'sigm'
            a{i} = sigmoid(W{i - 1} * a{i - 1} + b{i-1} * ones(1,m));
        case 'tanh_opt'
            a{i} = tanh_opt(W{i - 1} * a{i - 1} + b{i-1} * ones(1,m));
    end
end
switch options.output
    case 'sigm'
        a{n} =  sigmoid(W{n - 1} * a{n - 1} + b{n-1} * ones(1,m));
    case 'tanh_opt'
        a{n} = tanh_opt(W{n - 1} * a{n - 1} + b{n-1} * ones(1,m));
    case 'linear'
        a{n} =  W{n - 1} * a{n - 1} + b{n-1} * ones(1,m);
    case 'softmax'
        a{n} = W{n - 1} * a{n - 1};
        a{n} = exp(bsxfun(@minus, a{n}, max(a{n})));
        a{n} = bsxfun(@rdivide, a{n}, sum(a{n}));
end
%% backpropagation
% cost
% error = a{n} - label ;
error_no_wei = a{n} - label ;
error = bsxfun(@(w,e)w.*e,wei,error_no_wei);

switch options.output
    case {'sigm','linear','tanh_opt'}
        error_tmp = bsxfun(@(w,e)w.*e,sqrt(wei),error_no_wei);
        cost = sum( sum((error_tmp .* error_tmp))/(2*m) );
    case 'softmax'
        cost = -sum( sum(label .* log(a{n}))) ./ m;
end
if options.lambda
    for i=1 : n-1
        cost = cost + options.lambda / 2 * ( sum(sum(W{i} .* W{i})) );
    end
end
% grad
if nargout > 1
    switch options.output
        case 'sigm'
            d{n} =  error .* (a{n} .* (1 - a{n}));
        case 'tanh_opt'
            d{n} =  error .* (1 - a{n}.^2);
        case {'linear','softmax'}
            d{n} =  error;
    end
    for i = (n - 1) : -1 : 2
        % Derivative of the activation function
        switch options.activation_function
            case 'sigm'
                d_act = a{i} .* (1 - a{i});
            case 'tanh_opt'
                d_act = 1 - a{i}.^2;
        end
        
        % Backpropagate first derivatives
        d{i} = (W{i}' * d{i + 1}) .* d_act;
        
    end
    
    for i = 1 : (n - 1)
        dW{i} = d{i + 1} * a{i}'./m + options.lambda * W{i} ;
        db{i} = d{i + 1} * ones(m,1) ./m ;
    end
%     grad = [];
%     for i=1:n-1
%         grad = [grad ; dW{i}(:) ; db{i}(:)];
%     end
end
clear a b W b d d_act data error label;
cost = gather(cost);
for i=1:n-1
    dW{i} = gather(dW{i});
    db{i} = gather(db{i});
end
end
function sigm = sigmoid(z)
sigm = 1./(1 + exp(-z));
end
function tanh = tanh_opt(z)
tanh = (exp(z) - exp(-z)) ./ (exp(z) + exp(-z));
end