function [ cost , grad ] = CostGradComputing_weighted_gpu( theta , data , label ,netSizes , options, wei )
%COSTGRADCOMPUTING compute cost and grad of a single layer CCA
% wei should be row vector
%   Detailed explanation goes here
% tic;
% gpuDevice(1);
assert(nargin>=3, 'Insufficient input.\n');
if nargin<=3
    % default netSizes
    netSizes = [size(data1,1) 1000 500 100];
end
if nargin<5
    % default options
    options.output = 'sigm';
    options.activation_function = 'sigm';
    options.lambda = 0.001;
end
%% turn theta into matrix
n = numel(netSizes);   %layers' num
m = size(data , 2);    %sample num
W = cell(1 , n-1);
b = cell(1 , n-1);
endInx = 0;
for i=1:n-1
    startInx = endInx + 1;
    endInx = endInx + netSizes(i+1)*netSizes(i);
    
    W{i} = gpuArray(reshape(theta(startInx : endInx),netSizes(i+1),netSizes(i))); % gpu
    
    startInx = endInx + 1;
    endInx = endInx + netSizes(i+1);
    b{i} = gpuArray(theta(startInx : endInx));
end
assert(endInx == length(theta), 'Wrong Index .\n');
%% feedforward pass
a = cell(1,n);
a{1} = gpuArray(data);
Im = gpuArray(ones(1,m));

for i=2:n-1
    switch options.activation_function
        case 'sigm'
            a{i} = sigmoid(W{i - 1} * a{i - 1} + b{i-1} * Im);
        case 'tanh_opt'
            a{i} = tanh_opt(W{i - 1} * a{i - 1} + b{i-1} * Im);
    end
end
switch options.output
    case 'sigm'
        a{n} =  sigmoid(W{n - 1} * a{n - 1} + b{n-1} * Im);
    case 'tanh_opt'
        a{n} = tanh_opt(W{n - 1} * a{n - 1} + b{n-1} * ones(1,m));
    case 'linear'
        a{n} =  W{n - 1} * a{n - 1} + b{n-1} * ones(1,m);
    case 'softmax'
        a{n} = W{n - 1} * a{n - 1};
        a{n} = exp(bsxfun(@minus, a{n}, max(a{n})));
        a{n} = bsxfun(@rdivide, a{n}, sum(a{n}));
end
%% backpropagation
% cost
label = gpuArray(label);
wei   = gpuArray(wei);
error_no_wei = a{n} - label ;
error = bsxfun(@(w,e)w.*e,wei,error_no_wei);

% 
switch options.output
    case {'sigm','linear','tanh_opt'}
        error_tmp = bsxfun(@(w,e)w.*e,sqrt(wei),error_no_wei);
        cost = sum( sum((error_tmp .* error_tmp))/(2*m) );
%         cost = sum( sum((error .* error)) ./ (2*m));
    case 'softmax'
        cost = -sum( sum(label .* log(a{n}))) ./ m;
end
if options.lambda
    for i=1 : n-1
        cost = cost + options.lambda / 2 * ( sum(sum(W{i} .* W{i})) );
    end
end
% grad
if nargout > 1
    switch options.output
        case 'sigm'
            d{n} =  error .* (a{n} .* (1 - a{n}));
        case 'tanh_opt'
            d{n} =  error .* (1 - a{n}.^2);
        case {'linear','softmax'}
            d{n} =  error;
    end
    for i = (n - 1) : -1 : 2
        % Derivative of the activation function
        switch options.activation_function
            case 'sigm'
                d_act = a{i} .* (1 - a{i});
            case 'tanh_opt'
                d_act = 1 - a{i}.^2;
        end
        
        % Backpropagate first derivatives
        d{i} = (W{i}' * d{i + 1}) .* d_act;
        
    end
    
    for i = 1 : (n - 1)
        dW{i} = d{i + 1} * a{i}'./m + options.lambda * W{i} ;
        db{i} = d{i + 1} * ones(m,1) ./m ;
    end
    grad = [];
    for i=1:n-1
        grad = [grad ; dW{i}(:) ; db{i}(:)];
    end
end
cost = gather(cost);
grad = gather(grad);
% toc;
% fprintf('CostGradComputing with gpu: %f\n', toc);
end

function sigm = sigmoid_gpu(z)
x = gpuArray(z);
sigm = 1./(1 + exp(-x));
end
function sigm = sigmoid(z)
sigm = 1./(1 + exp(-z));
end
function tanh = tanh_opt(z)
tanh = (exp(z) - exp(-z)) ./ (exp(z) + exp(-z));
end
% function y = cuberoot(z , epsilon)
% y = z;
% while 1
%     y_old = y;
%     y = (2/3 * y.^3 + z)./(y .^2 + 1);
%     if sum(sum(y_old - y).^2) < epsilon
%         break;
%     end
% end
% end

